#! /usr/bin/python

""" Python Script to convert truetype fonts to 8x8, ans 16x16 bitmap font to be
used in dsgui. The truetype font should be monospaced, lest it turn out ugly. 

The output of the program is a C header file containing a unsigned char (8 bit)
vector of 255 tiles, each being 64bit for 8x8 or 256bit for 16x16 fonts. 

The format of the C header is compatible with the Nintendo DS Tile memory structure
and can therefore simply be copied into DS memory for usage. 

Python Packages Needed:
  Numpy - 1.0 or Later
  PIL -   Python Imaging Library 1.1.4 or Later

Usage:
  For Font 8:
  ttf2dsgui your_font.ttf 8

  For Font 16:
  ttf2dsgui your_font.ttf 16

Copyright: dsgui Project
"""
import ImageFont
import sys
from numpy import array, hstack, zeros

ft = ImageFont.truetype(sys.argv[1], int(sys.argv[2]))
sz_max = int(sys.argv[2])

#Arrays to hold our data
vpt = array([],'int32')

#Create ASCII Character list to extract
chars = []
for i in range(256):
  chars.append(chr(i))

#Convert TTF glyphs to array
for c in chars:
  #Get char size, check if fits
  sz = ft.getsize(c)
  if (sz[0] > sz_max or sz[1] > sz_max):
    print "Warning!, font was too large. Will Fail."

  #Get the bmp data from font
  a = array(ft.getmask(c),'int32') #Get BMP Character Map
  a[a != 0] = 5; #Set all non-zero values to 5. Paletted to be COLOR_TEXT in dsgui

  #If the current glyph is NOT a square, pad it so it is.
  if (sz[0] != sz_max or sz[1] != sz_max):
    #Turn vector a to a square
    a.resize(sz[1],sz[0]) #Turn 2D for easier code.
    b = zeros((sz_max, sz_max)) #Init Square
    for i in range(sz[1]):
      b[i][0:(sz[0])] = a[i]

    a = b #Done

  # Complete Vector of Characters, flattened
  vpt = hstack((vpt, a.flatten()))
  

#Now, Save data as a C header file
fn = open(sys.argv[1][:-4] + '_' + str(sz_max) + '.h', 'w')
fn.write('#ifndef __' + sys.argv[1][:-4] +'_H__\n')
fn.write('#define __' + sys.argv[1][:-4] +'_H__\n\n')

fn.write('int ' + sys.argv[1][:-4] + '_' + str(sz_max) + '_size = ' + str(sz_max*sz_max*256) + ';\n\n')
fn.write('u8 '+ sys.argv[1][:-4] + '_'  + str(sz_max) + '[' + str(sz_max*sz_max*256) + '] = { \n') #start header
fn.write(repr(list(vpt.astype('int8')))[1:-1]) #write data
fn.write('\n};\n #endif \n') #Close up the bracket.
fn.close()






