#include "dscontainer.h"

#include <string.h>

////////////////////////////////////
// CONSTRUCTORS AND INITIALIZATION
////////////////////////////////////

/*
 * Constructor that should be called from DSGui when creating the base containers
 *
 */
DSContainer::DSContainer( uchar screen ) {
  m_screen = screen;
  m_pos.x = 0; 
  m_pos.y = 0;
  m_dim.width = MAX_X; 
  m_dim.height = MAX_Y;
}

/*
 * Constructor for creating a container
 *
 */
DSContainer::DSContainer( DSContainer * p_parent, PPOSITION p_pos, PDIMENSIONS p_dim ) {
  m_p_parent = p_parent;
  
  memcpy(&m_pos, p_pos, sizeof(POSITION));
  memcpy(&m_dim, p_dim, sizeof(DIMENSIONS));
}

DSContainer::DSContainer( DSContainer * p_parent, PPOSITION p_pos, PDIMENSIONS p_dim, uint key_focus_left, uint key_focus_right ) {
  m_p_parent = p_parent;
  
  memcpy(&m_pos, p_pos, sizeof(POSITION));
  memcpy(&m_dim, p_dim, sizeof(DIMENSIONS));

  // Set Focus Keypresses
  // Cycle Left  
  if (key_focus_left == 0) 
    m_key_focus_left = KEY_DEFAULT_FOCUS_LEFT;
  else
    m_key_focus_left = key_focus_left;

  // Cycle Right
  if (key_focus_right == 0)
    m_key_focus_right = KEY_DEFAULT_FOCUS_RIGHT;
  else
    m_key_focus_right = key_focus_right;
}


////////////////////////////////////
// Class Member Functions
////////////////////////////////////

/*
 * This function adds a widget to the container. It keeps track of the DSWidget Linked List and ensures things are working as intended. 
 *
 */
RESULT DSContainer::AddDSWidget( DSWidget * p_widget, char widget_type ) {
  if (!p_widget)
    return INVALID_WIDGET;

  // Fill up the list of pointers accordingly.
  switch(widget_type) { // We want fall through.
    case TYPE_FOCUSABLE:
      if (m_p_first_focusable == NULL) {
        // If first, initialize all pointers
        m_p_first_focusable = (DSListenerWidget*)p_widget; // Check ?
        m_p_last_focusable = m_p_first_focusable;
        m_p_current_focusable = m_p_first_focusable;
      }
      else { // Add at end of list
        m_p_last_focusable->SetNextLW((DSListenerWidget*)p_widget); // Check ?
        m_p_last_focusable = (DSListenerWidget*)p_widget;
      } // Fall Through ->

    case TYPE_LISTENER:
      // If list is empty, simply start it
      if (m_p_first_lwidget == NULL)
        m_p_first_lwidget = (DSListenerWidget*)p_widget; // No Checking here? Might want to
      else {
        ((DSListenerWidget*)p_widget)->SetNextLW(m_p_first_lwidget);
        m_p_first_lwidget = (DSListenerWidget*)p_widget; // No Checking Here? Might want to 
      } // Fall Through ->

    case TYPE_WIDGET:
      // If list is empty, simply start it
      if (m_p_first_widget == NULL) 
        m_p_first_widget = p_widget;
      else { // Add at beginning of Widget List
        p_widget->SetNext(m_p_first_widget);
        m_p_first_widget = p_widget; }
  }

  return SUCCESS;
}

RESULT DSContainer::ClickEvent( PPOSITION p_pos ) {
  POSITION p_new_pos; // Create new position to avoid changing the one passed.
  DSListenerWidget * p_current = m_p_first_lwidget;

  // Check If we were clicked
  if (!CheckClickEvent(p_pos))
    return MOUSE_EVENT_NOT_CLICKED;

  // Modify position so that whatever we send is relative to us.
  p_new_pos.x = p_pos->x - m_pos.x;
  p_new_pos.y = p_pos->y - m_pos.y;

  // Iterate through all DSListeners to find which was clicked.
  while(p_current->ClickEvent( &p_new_pos )) {// While MOUSE_EVENT_NOT_CLICKED
    if (p_current->GetNextLW(p_current)) // If getting next item gives END_OF_LIST
      break;
  }

  // Since we were clicked, return clicked wether we found a viable Listener or not.
  // This will prevent looping through all item in the main loop since we know cursor
  // was here and found nothing.
  return MOUSE_EVENT_CLICKED;
}

RESULT DSContainer::Draw( ) {
  return STUB_FUNCTION;
}

// Shifts the focus from its corrent location to the next focusable object
// If no next object exist or error retrieving, return END_OF_LIST
RESULT DSContainer::FocusNext() {
  DSListenerWidget *p_tmp = NULL;

  if (m_p_current_focusable == m_p_last_focusable)
    return END_OF_LIST;

  if (m_p_current_focusable->GetNextFocusLW(p_tmp))
    return END_OF_LIST;

  m_p_current_focusable = p_tmp;
  return SUCCESS;
    
}

// Shifts the focus from its corrent location to the previous focusable object.
// If not previous object exist or error retrieving, returns END_OF_LIST
RESULT DSContainer::FocusPrev() {
  DSListenerWidget *p_tmp = NULL;

  if (m_p_current_focusable == m_p_first_focusable)
    return END_OF_LIST;

  if (m_p_current_focusable->GetPrevFocusLW(p_tmp))
    return END_OF_LIST;

  m_p_current_focusable = p_tmp;
  return SUCCESS;
}


RESULT DSContainer::KeyPressEvent( uint key_pressed ) {
  return STUB_FUNCTION;
}


