#include "dspainter.h"


DSPainter::DSPainter(int screen, PDSTHEME p_theme): m_p_theme(p_theme) {
  /* We have to set up all memory addresses for the tiled interface. 
   * BG0 has the BG_MAIN tiles, and is 16kB. (Used-Defined Tiles would be here)
   * BG1 has the BG_TEXT tiles, and is 16kB. (Fulled used by 8x8 Font)
   * BG2 is not yet in use. 19/06/07 - sberube
   * BG3 is not yet in use. 19/06/07 - sberube
   */
  if (screen == SCREEN_MAIN) {
    m_p_screen = &DISPLAY_CR;

    m_p_map_mem[BG_MAIN] =  (uint16*)BG_MAP_RAM(0); // Starts at 0x06000000 each base 2k
    m_p_map_mem[BG_TEXT] =  (uint16*)BG_MAP_RAM(1); // Starts at 0x06000800

    m_p_tile_mem[BG_MAIN] =  (uint8*)BG_TILE_RAM(1); // Starts at  0x06004000
    m_p_tile_mem[BG_TEXT] =  (uint8*)BG_TILE_RAM(2); // Starts at  0x06008000

    m_p_pal_mem  =  BG_PALETTE; // ((uint16*)0x05000000)
  
    m_p_bg[BG_MAIN] = &BG0_CR; //0x04000008
    m_p_bg[BG_TEXT] = &BG1_CR; //0x0400000A

  } else {
    m_p_screen = &SUB_DISPLAY_CR;

    m_p_map_mem[BG_MAIN] =  (uint16*)BG_MAP_RAM_SUB(0); // Starts at 0x06200000 each base 2k
    m_p_map_mem[BG_TEXT] =  (uint16*)BG_MAP_RAM_SUB(1); // Starts at 0x06200800

    m_p_tile_mem[BG_MAIN] =  (uint8*)BG_TILE_RAM_SUB(1); // Starts at  0x06204000
    m_p_tile_mem[BG_TEXT] =  (uint8*)BG_TILE_RAM_SUB(2); // Starts at  0x06208000

    m_p_pal_mem = BG_PALETTE_SUB;

    m_p_bg[BG_MAIN] = &SUB_BG0_CR; //0x04001008
    m_p_bg[BG_TEXT] = &SUB_BG1_CR; //0x0400100A
  }

  m_max_x = 32;
  m_max_y = 32;
  *m_p_bg[BG_MAIN] = BG_32x32 | BG_COLOR_256 | BG_MAP_BASE(0) | BG_TILE_BASE(1) | BG_PRIORITY(1);
  *m_p_bg[BG_TEXT] = BG_32x32 | BG_COLOR_256 | BG_MAP_BASE(1) | BG_TILE_BASE(2) | BG_PRIORITY(0); //Text Highest Priority
  *m_p_screen = MODE_3_2D | DISPLAY_BG0_ACTIVE | DISPLAY_BG1_ACTIVE; // 0x10003
  
  initPalette();
  initTiles();
  initMap();
}

DSPainter::~DSPainter() {
}

RESULT DSPainter::initTiles() {
  int i;

  // Create Base Tiles
  memset(&m_p_tile_mem[BG_MAIN][TILE_EMPTY*64], 0, 64); // Background (empty) tile
  memset(&m_p_tile_mem[BG_MAIN][TILE_SOLID*64], COLOR_1, 64); // Solid Tile

  // Create the gradient tile
  memset(&m_p_tile_mem[BG_MAIN][TILE_GRAD*64], COLOR_2, 8); // First Row
  for (i=0; i< 6; i++) memset(&m_p_tile_mem[BG_MAIN][TILE_GRAD*64] + (i+1)*8, COLOR_GRAD_START + i, 8); // Mid Rows
  memset(&m_p_tile_mem[BG_MAIN][TILE_GRAD*64] + 7*8, COLOR_1, 8); // Last Row

  // Copy Font in Memory of BG_TEXT
  if(m_p_theme->font_small == NULL)
    return INVALID_THEME_FONT_SMALL;

  swiCopy(m_p_theme->font_small, m_p_tile_mem[BG_TEXT], m_p_theme->font_small_size); // 8k half words, or 16kB
  memset(&m_p_tile_mem[BG_TEXT][TILE_EMPTY*64], 0, 64);

  return SUCCESS;
}

RESULT DSPainter::initPalette() {
  int R,G,B, DR,DG,DB, i;

  // Save main colors in palette
  for (i = 0; i< NUM_THEME_COLORS; i++)
    m_p_pal_mem[i+1] = m_p_theme->colors[i];

  // Create the 6 Gradient Colors
  R = m_p_theme->colors[1] & 0x1F;
  G = m_p_theme->colors[1] >> 5 & 0x1F;
  B = m_p_theme->colors[1] >> 10 & 0x1F;

  DR = (m_p_theme->colors[0] & 0x1F)  - R;
  DG = (m_p_theme->colors[0] >> 5 & 0x1F) - G;
  DB = (m_p_theme->colors[0] >> 10 & 0x1F) - B;

  for (i = 0; i<6; i++) 
    m_p_pal_mem[COLOR_GRAD_START + i] = RGB15(R + (DR/6)*(i+1), G + (DG/6)*(i+1), B + (DB/6)*(i+1));

  return SUCCESS;
}

RESULT DSPainter::initMap() {
  return SUCCESS;
}


/* 
 * Public Functions
 */
RESULT DSPainter::BGClearRect(u8 x, u8 y, u8 width, u8 height) {
  return MapRect(x,y,width,height, TILE_EMPTY);
}

RESULT DSPainter::BGDrawRect(u8 x, u8 y, u8 width, u8 height) {
  return MapRect(x,y,width,height, TILE_SOLID);
}

RESULT DSPainter::BGDrawGradRect( u8 x, u8 y, u8 width, u8 height) {
  int i,j;

  if (x+width > m_max_x || y+height > m_max_y)
    return POSITION_OUT_OF_BOUNDS;

  for(i=1; i< height-1; i++) 
    for(j=0; j< width; j++)
      m_p_map_mem[BG_MAIN][(y+i)*m_max_x + x + j] = TILE_SOLID;
  
  for(i=0; i< width; i++) {
    m_p_map_mem[BG_MAIN][y*m_max_x + x + i] = TILE_GRAD; // First Row, Gradient Tile
    if (height > 1)
      m_p_map_mem[BG_MAIN][(y+height-1)*m_max_x + x + i] = TILE_GRAD | (1 << 11); // Last Row, VErtical Flip Gradient Tile
  }

  return SUCCESS;
}

RESULT DSPainter::BGDrawText( u8 x, u8 y, char * text ) {
  uint i;

  for(i=0; i< strlen(text); i++)
    m_p_map_mem[BG_TEXT][y*m_max_x + x + i] = text[i];

  if (x+strlen(text) > m_max_x || y > m_max_y)
    return POSITION_OUT_OF_BOUNDS;

  return SUCCESS;
}

/*
 * Private (helper) Functions
 */
RESULT DSPainter::MapRect( u8 x, u8 y, u8 w, u8 h, uint16 tile) {
  int i,j;

  for(i=0; i< h; i++) 
    for(j=0; j< w; j++)
      m_p_map_mem[BG_MAIN][(y+i)*m_max_x + x + j] = tile;

  if (x+w > m_max_x || y+h > m_max_y)
    return POSITION_OUT_OF_BOUNDS;

  return SUCCESS;

}

