#ifndef __DS_PAINTER__
#define __DS_PAINTER__
// TODO: Focus Sprites Support, + Bounding box sprite ? Or Corner Tiles ? Sprites probably better. (For animations)
// TODO: Add Support for Custom Tiles object (Icons, etc..)
// TODO: Add x point sprite font support?

#include <string.h> //memset, strlen
#include "dsobject.h"

/*
 * Theme, Palette, and Define Constants
 */
typedef struct _DSTHEME {
  uint16  colors[5];
  u8  *   font_small;
  int     font_small_size;
} DSTHEME, *PDSTHEME;

/*
 * Define Theme Colors
 * NOTE: The first 16 colors are used for main colors
 *     1-16 - Main Color Palette
 *     17-22 - Gradient Palette for Gradient Functions (Gradient is always 6 colors)
 */
#define NUM_THEME_COLORS  5
#define COLOR_1     1
#define COLOR_2     2
#define COLOR_3     3
#define COLOR_4     4
#define COLOR_TEXT  5
#define COLOR_GRAD_START  17  // Grad is a gradient from Color 2 to Color 1

// Define tile names here for all tiles used in painting the gui
#define TILE_EMPTY  0
#define TILE_SOLID  1
#define TILE_GRAD   2

// Define Background, Text8, etc.. Indices for pointers
#define BG_NB_USED    2

#define BG_MAIN 0
#define BG_TEXT 1

/*
 * DSPainter Class Definition
 */
class DSPainter: public DSObject {
  protected:
    u8 m_max_x, m_max_y; // Tile Map Dimensions

    // Memory Initialization BG
    RESULT initTiles();
    RESULT initPalette();
    RESULT initMap();

    // Memory Initialization SP
    RESULT initSprites();
    RESULT initSPPalette();
    
    // DS BG Memory Locations, init in constructor
    vuint32 * m_p_screen;
    uint16  * m_p_pal_mem;
    vuint16 * (m_p_bg[BG_NB_USED]);
    u8      * (m_p_tile_mem[BG_NB_USED]);
    uint16  * (m_p_map_mem[BG_NB_USED]);

    // DS Sprite Memory Locations, init in constructor
    SpriteEntry * m_p_sp;
    SpriteRotation * m_p_sprots;

    uint16 * m_p_sp_mem;
    uint16 * m_p_sp_mem_next;
    

    PDSTHEME m_p_theme;// Theme

  public:
    DSPainter(int screen, PDSTHEME p_theme);
    ~DSPainter();
  
    // Background Functions
    RESULT BGClearRect(u8 x, u8 y, u8 width, u8 height);
    RESULT BGDrawGradRect(u8 x, u8 y, u8 width, u8 height);
    RESULT BGDrawRect(u8 x, u8 y, u8 width, u8 height);
    RESULT BGDrawText(u8 x, u8 y, char * text);

    // Sprite Functions
    RESULT SPDrawBox(u8 x, u8 y, u8 width, u8 height);

    // Getters and Setters
    RESULT ShiftBGMain(PPOSITION result);
    RESULT ShiftBGText(PPOSITION result);
    RESULT GetBGMainShift(PPOSITION result);
    RESULT GetBGTextShift(PPOSITION result);
    
  private:
    RESULT MapRect(u8 x, u8 y, u8 w, u8 h, uint16 tile);
};



#endif
