#include "dswidget.h"

#include <string.h>  /* for memcpy */

RESULT DSWidget::GetDimensions( PDIMENSIONS p_dims ) {
  p_dims = &m_dim;
  return SUCCESS;
}

RESULT DSWidget::GetNext( DSWidget * p_dswidget ) {
  if (m_p_next_widget == NULL)
    return END_OF_LIST;

  p_dswidget = m_p_next_widget;
  return SUCCESS;
}

RESULT DSWidget::GetParent( DSWidget * p_dswidget ) {
  if (m_p_parent == NULL)
    return END_OF_LIST;

  p_dswidget = m_p_parent;
  return SUCCESS;
}

RESULT DSWidget::GetPosition( PPOSITION p_pos ) {
  p_pos = &m_pos;
  return SUCCESS;
}

char DSWidget::GetState() { 
  return m_state; 
}

// #DEV_ITEM - Not sure we should check dimensions here? - Changed OUT_OF_BOUNDS to warning.
// This will check the dimensions to make sure they are not out of bounds
// with the parent object. If parent object is NULL, DS Screen dimensions
// will be used as maximum values.
// RETURNS:
//  POSITION_OUT_OF_BOUNDS, as warning
//  SUCCESS, everything ok
RESULT DSWidget::SetDimensions( PDIMENSIONS p_dim ) {
  uchar max_x = MAX_X;
  uchar max_y = MAX_Y;
  
  memcpy(&m_dim, p_dim, sizeof(DIMENSIONS));
  // If parent set, get dimensions of parents as max size values
  if (m_p_parent != NULL) {
    PDIMENSIONS p_pdims = NULL;
    m_p_parent->GetDimensions( p_pdims ); //This can't fail
    max_x = p_pdims->width;
    max_y = p_pdims->height; 
  }

  // check that dimensions aren't out of bounds
  if (p_dim->width + m_pos.x > max_x || p_dim->height + m_pos.y > max_y)
    return DIMENSIONS_OUT_OF_BOUNDS;

  return SUCCESS;
}

// Set the next widget in line for the list
RESULT DSWidget::SetNext( DSWidget * p_widget ) {
  m_p_next_widget = p_widget;
  return SUCCESS;
}

RESULT DSWidget::SetParent( DSWidget * p_dswidget ) {
  if (p_dswidget == NULL)
    return INVALID_WIDGET;

  m_p_parent = p_dswidget;

  // Check if we fit in new parent #DEV_ITEM - Good Design? Should remove ?
  if (this->SetPosition(&m_pos)) {
    // Didn't fit, set to fill parent
    PDIMENSIONS p_dims = NULL;

    p_dswidget->GetDimensions(p_dims); // Can't fail

    //Dimensions were out of bound
    m_pos.x = 0; m_pos.y = 0;
    memcpy(&m_dim, p_dims, sizeof(DIMENSIONS));
  }

  return SUCCESS;
}

// This will check the position to make sure they are not out of bounds
// with the parent object. If parent object is NULL, DS Screen dimensions
// will be used as maximum values.
// RETURNS:
//  POSITION_OUT_OF_BOUNDS, as warning
//  SUCCESS, everything ok
RESULT DSWidget::SetPosition( PPOSITION p_pos ) {
  uchar max_x = MAX_X;
  uchar max_y = MAX_Y;

  memcpy(&m_pos, p_pos, sizeof(POSITION));

  if (m_p_parent != NULL) {
    PDIMENSIONS p_pdims = NULL;
    m_p_parent->GetDimensions( p_pdims ); //This can't fail
    max_x = p_pdims->width;
    max_y = p_pdims->height; 
  }

  // only need to check y since x is limited by type
  if (p_pos->y > max_y)
    return POSITION_OUT_OF_BOUNDS;

  // check that new position doesn't put dimensions out of bounds
  if (p_pos->x + m_dim.width > max_x || p_pos->y + m_dim.height > max_y)
    return DIMENSIONS_OUT_OF_BOUNDS;

  return SUCCESS;
}

RESULT DSWidget::SetState( char state ) { 
  if (state != STATE_ACTIVE || state != STATE_INACTIVE || state != STATE_OFF)
    return INVALID_STATE;

  m_state = state; 
  return SUCCESS;
}

